#!/usr/bin/env node
'use strict';
var path = require('path');

var Elapsed = require('elapsed');
var antwar = require('antwar');
var objectMerge = require('object-merge');
var program = require('commander');

require('es6-promise').polyfill();
require('promise.prototype.finally');

var lib = require('./lib');
var version = require('./package.json').version;


main();

function main() {
    var now = new Date();
    var defaultConfig = {
        site: {
            port: 3000,
            output: 'build',
            boilerplate: 'antwar-boilerplate',
            deploy: {
                branch: 'gh-pages',
            },
            // it's not possible to point at console directly here
            // that will cause a circular reference error when merging objects
            console: {
                info: function() {
                    console.info.apply(console, arguments);
                },
                error: function() {
                    console.error.apply(console, arguments);
                },
                warn: function() {
                    console.warn.apply(console, arguments);
                },
            }
        }
    };

    program.version(version).
        option('-c, --config <file>', 'Configuration file (defaults to antwar.config.js)').
        option('-i, --init <directory>', 'Initialize a project').
        option('-b, --build', 'Build site').
        option('-s, --serve [port]', 'Serve site. Port (defaults to ' +
            defaultConfig.port + ')', parseInt).
        option('-g, --ghpages', 'Deploy to gh-pages').
        option('-d, --develop', 'Open a browser in development mode');

    program.parse(process.argv);

    // TODO: this would be a good place to validate configuration (push to core)
    // + show warnings about possible misspellings etc.
    var config = getConfig(
        defaultConfig,
        path.join(process.cwd(), program.config || 'antwar.config.js')
    );

    // XXX: having console in defaultConfig causes Circular reference error at object merge
    // so we need to patch it a little
    if(!config.site.console) {
        config.site.console = console;
    }

    if(program.init) {
        console.log('Starting to initialize project');

        config.site.output = program.init;

        lib.init(config.site).finally(function() {
            showElapsedTime(now);
        });
    }
    else if(program.build) {
        console.log('Starting to build');

        antwar.build(config).then(function() {
            console.log('Build finished');
        }).catch(function(err) {
            console.error('Build failed', err);
        }).finally(function() {
            showElapsedTime(now);
        });
    }
    else if(program.serve) {
        config.site.port = parseInt(program.serve) || config.site.port;

        lib.serve(config.site);
    }
    else if(program.ghpages) {
        console.log('Starting to deploy');

        lib.deploy(config.site).catch(function(err) {
            console.error('Deployment failed', err);
        }).finally(function() {
            showElapsedTime(now);
        });
    }
    else if(program.develop) {
        console.log('Starting development build');

        antwar.develop(config).then(function() {
            console.log('Development build finished');
        }).catch(function(err) {
            console.error('Development build failed', err);
        }).finally(function() {
            showElapsedTime(now);
        });
    }
}

function showElapsedTime(a, b) {
    console.log('\nTime elapsed:', (new Elapsed(a, b || new Date())).optimal);
}

function getConfig(defaultConfig, defaultPath) {
    try {
        var loadedConfig = require(defaultPath);

        return objectMerge(defaultConfig, loadedConfig);
    } catch(e) {
        console.error(e);
    }

    return defaultConfig;
}
